"""
The metaclass 'KeeMeta' automatically creates the KeeNum class for which
the class here is a placeholder of sorts.

"""
#  AGPL-3.0 license
#  Copyright (c) 2025 Asger Jon Vistisen
from __future__ import annotations

from . import KeeMeta

KeeNum = KeeMeta.getCoreKeeNum()
